/* Author : Scott Holden ( scotth@thezone.net )
 * 
 * Pop3 Email checker.
 *
 * Last Updated : Mar 20, 1999
 *
 */


#include "Pop3Client.h"
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

Pop3 pop3Create(void){

    Pop3 pc;

    pc = (Pop3)malloc( sizeof(*pc) );
    if( pc == 0)
        return 0;
    strcpy(pc->password , "");
    strcpy(pc->userName , "");
    pc->connected     = NOT_CONNECTED;
    pc->serverPort    = 110;
    pc->localPort     = 0;
    pc->numOfMessages = 0;
    pc->numOfUnreadMessages = 0;
    pc->inBufSize     = 0;
    pc->outBufSize    = 0;

    return pc;
}
int pop3MakeConnection(Pop3 pc, char *serverName, int port){

    pc->s = socket(AF_INET, SOCK_STREAM, 0 );
    memset( &(pc->server), 0 , sizeof(pc->server));
    pc->server.sin_family = AF_INET;
    pc->hp = gethostbyname(serverName);
    if( pc->hp == 0)
        return -1;
    memcpy( &(pc->server.sin_addr), pc->hp->h_addr, pc->hp->h_length);
    pc->server.sin_port = htons(port);
    if ( connect(pc->s, (struct sockaddr *)&(pc->server)
                               , sizeof(pc->server)) < 0 )
        return -1;
    pc->connected = CONNECTED;
   return 0;
}

int pop3IsConnected(Pop3 pc){

    if( pc->connected == CONNECTED)
        return 1;
    return 0;
}

int pop3Login(Pop3 pc, char *name, char *pass){

      int size; 
      char temp[1024];

      if( pc->connected == NOT_CONNECTED ){
          perror("Not Connected To Server\n");
          return -1;
      }

      size = recv(pc->s,&pc->inBuf,1024,0);
      memset(temp,0,1024);
      memcpy(temp,pc->inBuf,size);
      if( temp[0] != '+' ){
          fprintf(stderr,"Error Logging in\n");
          return -1;
      }

      sprintf(pc->outBuf,"USER %s\r\n",name);
      send(pc->s, &pc->outBuf,strlen(pc->outBuf),0); 
      size =recv(pc->s,pc->inBuf,1024,0);
      memset(temp,0,1024);
      memcpy(temp,pc->inBuf,size);
      if( temp[0] != '+' ){
          fprintf(stderr,"Invalid User Name\n");
          return -1;
      }

      memset(pc->outBuf,0,1024);
      sprintf(pc->outBuf,"PASS %s\r\n",pass);
      send(pc->s, pc->outBuf, strlen(pc->outBuf),0 );
      size =recv(pc->s,&pc->inBuf,1024,0);
      memset(temp,0,1024);
      memcpy(temp,pc->inBuf,size);
      if( temp[0] != '+' ){
          fprintf(stderr,"Incorrect Password\n");
          return -1;
      }

      return 0;
}

int	store_from(char *buf, int pos, int buflen, int num, Pop3 pc) {
  int	end;
  int	deb_addr;
  int	fin_addr;
  int	i;
  
  /* strip the white spaces */
  for (; (pos < buflen) && isspace(buf[pos]); pos++);
  /* get the end of the line */
  for (end = pos; (end + 1 < buflen) && 
	 (buf[end] != '\r') && (buf[end + 1] != '\n'); end++);
  /* try to find the '@' sign for the address */
  for (i = pos; (i < end) && buf[i] != '@'; i++);
  if (i < end) { /* we found a @ sign before the end of the line */
    for (deb_addr = i; deb_addr > pos && !isspace(buf[deb_addr]); deb_addr--);
    while (isspace(buf[deb_addr]) || ('<' == buf[deb_addr]))
      deb_addr++;
    for (fin_addr = i; fin_addr < end && !isspace(buf[fin_addr]); fin_addr++);
    if ('>' == buf[fin_addr - 1])
      fin_addr--;
    memcpy(pc->mails[num].from, buf + deb_addr, 
	   (fin_addr - deb_addr > MAIL_BUFLEN) ? 
	   MAIL_BUFLEN : fin_addr - deb_addr);
  }
  else {
    memcpy(pc->mails[num].from, buf + pos, 
	   (end - pos > MAIL_BUFLEN) ? MAIL_BUFLEN : end - pos);
  }
  return (end + 2);
}

int	store_subject(char *buf, int pos, int buflen, int num, Pop3 pc) {
  int	end;
  int	i;

  /* get the end of the line */
  for (end = pos, i = 0; (i < MAIL_BUFLEN) && (end + 1 < buflen) && 
	 (buf[end] != '\r') && (buf[end + 1] != '\n'); end++, i++);
  for (; (pos < buflen) && (buf[pos] == ' '); pos++);
  memcpy(pc->mails[num].subject, buf + pos, end - pos);
  return (end + 2);
}

int	parse_header(char *buf, int buflen, int num, Pop3 pc) {
  int	pos = 0;
  int	retour = 1;
  
  /* parse the header for the from and subject fields */
  while (pos < buflen) {
    if ((buflen > (pos + 2)) && !strncmp(".\r\n", buf + pos, 3)) {
      /* this is the end */
      pos = buflen;
      retour = 0;
    }
    else if ((buflen > (pos + 4)) && !strncmp("From:", buf + pos, 5)) {
      pos += 5;
      pos = store_from(buf, pos, buflen, num, pc);
    }
    else if ((buflen > (pos + 7)) && !strncmp("Subject:", buf + pos, 8)) {
      pos += 8;
      pos = store_subject(buf, pos, buflen, num, pc);
    }
    else {
      for( ; (buflen > (pos + 2)) && (buf[pos] != '\r') && 
	     (buf[pos + 1] != '\n'); pos++);
      pos += 2;
    }
  }
  return (retour);
}

int	pop3VerifStats(Pop3 pc)
{
  int	retour = 0;
  char	*ptr;
  int	size;
  
  if( pc->connected == NOT_CONNECTED )
    return (-1);
  
  /* Find total number of messages in mail box */
  sprintf(pc->outBuf,"STAT\r\n");
  send(pc->s, pc->outBuf, strlen(pc->outBuf),0 );
  size = recv(pc->s,pc->inBuf,1024,0);
  if( pc->inBuf[0] != '+' ){
    perror("Error Receiving Stats");
    return (-1);
  }
  ptr = strtok(pc->inBuf, " ");
  ptr = strtok( 0," ");
  ptr = strtok( 0, " ");
  if (ptr)
    retour = atoi(ptr); 
  return (retour);
}

int	pop3CheckMail(Pop3 pc) {
  int	size;
  char	temp[4096];
  char	*ptr;
  int	i;
  int	pass;
  int	go;
  
  if( pc->connected == NOT_CONNECTED )
    return -1;
  
  /* Find total number of messages in mail box */
  sprintf(pc->outBuf,"STAT\r\n");
  send(pc->s, pc->outBuf, strlen(pc->outBuf),0 );
  size =recv(pc->s,pc->inBuf,1024,0);
  memset(temp,0,1024);
  memcpy(temp,pc->inBuf,size);
  ptr = strtok(temp, " ");
  ptr = strtok( 0," ");
  pc->numOfMessages = atoi(ptr);
  ptr = strtok( 0, " ");
  pc->sizeOfAllMessages = atoi(ptr); 
  if( temp[0] != '+' ){
    perror("Error Receiving Stats");
    return -1;
  }
  
  sprintf(pc->outBuf,"LAST\r\n");
  send(pc->s, pc->outBuf, strlen(pc->outBuf),0 );
  size = recv(pc->s,pc->inBuf,1024,0);
  memset(temp,0,1024);
  memcpy(temp,pc->inBuf,size);
  ptr = strtok(temp, " ");
  ptr = strtok( 0," ");
  pc->numOfUnreadMessages = pc->numOfMessages - atoi(ptr);
  
  if( temp[0] != '+' ){
    perror("Error Receiving Stats");
    return -1;
  }
  
  /* get the subject and From fields of MAX_NB_MSG mail */
  for (i = 0; (i < MAX_NB_MSG) && (i < pc->numOfMessages); i++) {
    /* reset the from and subject fields for num */
    memset(&pc->mails[i], 0, sizeof(t_mail));
    sprintf(pc->outBuf, "TOP %d 0\r\n", i + 1);
    send(pc->s, pc->outBuf, strlen(pc->outBuf), 0);
    pass = 0;
    go = 1;
    while (go && (0 < (size = recv(pc->s,pc->inBuf,4096,0)))) {
      memset(temp, 0, 4096);
      memcpy(temp, pc->inBuf, size);
      if (!pass && (temp[0] != '+')) {
	fprintf(stderr, "Error while getting TOP %d 0\n", i + 1);
	go = 0;
	/* must set the from and subject fields to correct value */
	strncpy(pc->mails[i].from, "::error::", 9);
	continue;
      }
      pass++;
      go = parse_header(temp, size, i, pc);
    }
  }
  
  return 1;
}     

int	pop3DeleteMail(int num, Pop3 pc) 
{
  int	size;

  sprintf(pc->outBuf, "DELE %d\r\n", num);
  send(pc->s, pc->outBuf, strlen(pc->outBuf), 0);
  size = recv(pc->s, pc->inBuf, 4096, 0);
  if ('+' != pc->inBuf[0]) {
    perror("error while deleting mail");
    return (1);
  }
  return (0);
}
  

  

int  pop3GetTotalNumberOfMessages( Pop3 pc ){
     if( pc != 0 )
         return pc->numOfMessages;
     return -1;    
}

int  pop3GetNumberOfUnreadMessages( Pop3 pc ){
    if( pc != 0)
        return pc->numOfUnreadMessages;
    return -1;
}

int pop3Quit(Pop3 pc){
    int size;
    if( pc->connected == NOT_CONNECTED )
        return -1;
    send(pc->s, "quit\r\n", 6,0 );
    size =recv(pc->s,&pc->inBuf,1024,0);
    pc->connected = NOT_CONNECTED;
    if(pc->s != 0)
        close(pc->s);
    return 0;

}



