/*
 *  wmpop3.c
 *  pop3 mail checker. 
 *  by Scott Holden (scotth@thezone.net)
 *
 *  Contains some code from WMInet Dockable app.
 *             - BlitNum and BlitString
 *
*/


#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>

#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>

#include <X11/xpm.h>
#include <X11/Xlib.h>
#include <X11/extensions/shape.h>

#include "../wmgeneral/misc.h"
#include "../wmgeneral/wmgeneral.h"
#include "Pop3Client.h"

#include "wmpop3.xpm"

char wminet_mask_bits[64*64];
int  wminet_mask_width = 64;
int  wminet_mask_height = 64;


#define WMPOP3_VERSION "1.0"

#define CHAR_WIDTH 5
#define CHAR_HEIGHT 5
#define	TOP_MARGIN 5
#define	LEFT_MARGIN 4
#define	NUMBERS	93
#define	LETTERS	85
#define SMALL_NUM 76
#define	SN_CHAR_W 3
#define	NB_DISP	8
#define	NB_LINE	7
#define SEC_IN_MIN 60
#define YES 1
#define NO 0

char *ProgName;

char mailclient[32]   = "pine";
char password[32];
char username[32];
char popserver[128];
int  serverport       = 110;
int  mailCheckDelay   = 10;  /* default */
int  autoChecking     = YES; /* default */
int  newMessagesOnly  = YES; /* default */
char config_file[256] = "not-defined";

void usage(void);
void printversion(void);
void wmCheckMail_routine(int, char **);
int  readConfigFile( char *filename );

void BlitString(char *name, int x, int y);
void BlitNum(int num, int x, int y, int todelete);

int	pop3DeleteMail(int num, Pop3 pc);
int	pop3VerifStats(Pop3 pc);

/********************************
 *           Main Program
 ********************************/
int main(int argc, char *argv[]) {

	int		i;

	ProgName = argv[0];
	if (strlen(ProgName) >= 5)
		ProgName += (strlen(ProgName) - 5);
	
	for (i=1; i<argc; i++) {
		char *arg = argv[i];

		if (*arg=='-') {
			switch (arg[1]) {
			case 'd' :
				if (strcmp(arg+1, "display")) {
					usage();
					exit(1);
				}
				break;
			case 'g' :
				if (strcmp(arg+1, "geometry")) {
					usage();
					exit(1);
				}
				break;
			case 'v' :
				printversion();
				exit(0);
				break;
            case 'c' :
                if (argc > (i+1))
                {
                    strcpy(config_file, argv[i+1]);
                    i++;
                }
                break;
            default:
				usage();
				exit(0);
				break;
			}
		}
	}

	wmCheckMail_routine(argc, argv);

	return 0;
}

void		build_line(char *buf, int num, int index, Pop3 pc) 
{
  int		len;
  int		pos;
  char		tmp[256];
  
  memset(tmp, 0, sizeof(char) * 256);
  for (len = 0; len < MAIL_BUFLEN && pc->mails[num].from[len]; len++);
  memcpy(tmp, pc->mails[num].from, len);
  tmp[len] = '/';
  pos = len + 1;
  for (len = 0; len < MAIL_BUFLEN && pc->mails[num].subject[len]; len++);
  memcpy(tmp + pos, pc->mails[num].subject, len);
  len += pos;
  tmp[len++] = ' ';
  tmp[len++] = '*';
  tmp[len++] = ' ';
  tmp[len] = '\0';
  pos = index % len;
  if (len - pos < NB_DISP) { /* We are at the end of the string */
    memcpy(buf, tmp + pos, len - pos);
    memcpy(buf + len - pos, tmp, NB_DISP - len + pos);
  }
  else {
    fflush(stdout);
    memcpy(buf, tmp + pos, NB_DISP);
  }
}

/* display the mail number in the left column */
void		display_index(int index_vert, int totalMessages, Pop3 pc)
{
  int		i;
  
  for (i = index_vert; i < (NB_LINE + index_vert) && 
	 i < MAX_NB_MSG && i < totalMessages; i++) {
    BlitNum(i + 1, LEFT_MARGIN, ((i - index_vert) * 6) + TOP_MARGIN,
	    pc->mails[i].todelete);
  }
}

/* return < 0 if error
   0 if no pb
   else nb of mails which couldn't be deleted */
int	DeleteMail(Pop3 pc)
{
  int	etat = 0;
  int	old_sizeOfAllMessages;
  int	i;

  old_sizeOfAllMessages = pc->sizeOfAllMessages;
  etat = pop3MakeConnection(pc,popserver,serverport);
  if (-1 != etat) {
    BlitString("login", 10, TOP_MARGIN);
    RedrawWindow();
    etat = pop3Login(pc, username,password);
  }
  if (-1 != etat) {
    BlitString("chk cont", 10, 6 + TOP_MARGIN);
    RedrawWindow();
    etat = pop3VerifStats(pc);
  }
  if (-1 != etat) {
    if (etat != old_sizeOfAllMessages) {
      BlitString("mailbox", 10, TOP_MARGIN);
      BlitString("content", 10, 6 + TOP_MARGIN);
      BlitString("changed", 10, 6*2 + TOP_MARGIN);
      BlitString("between", 10, 6*3 + TOP_MARGIN);
      BlitString("updates", 10, 6*4 + TOP_MARGIN);
      BlitString("del can.", 10, 6*6 + TOP_MARGIN);
      RedrawWindow();
      sleep(1);
      for (i = 0, etat = 0; i < pc->numOfMessages && i < MAX_NB_MSG; i++) {
	etat += pc->mails[i].todelete;
      }
    }
    else {
      etat = 0;
      for (i = 0; i < pc->numOfMessages && i <MAX_NB_MSG; i++) {
	if (pc->mails[i].todelete) {
	  etat += pop3DeleteMail(i + 1, pc);
	}
      }
    }
  }
  pop3Quit(pc);
  return (etat);
}  



/*
 * wmCheckMail_routine : This function is used to set up the X display
 *           for wmpop3 and check for mail every n number of minutes.
 */

void		wmCheckMail_routine(int argc, char **argv){

    int		mailWaiting    = 0;
    int		totalMessages  = 0;
    long	startTime      = 0;
    long	nextCheckTime  = 0;
    int		forcedCheck    = NO;
    int		buttonStatus   = -1;
    int		i, j;
    XEvent	Event;
    Pop3	pc;
    char	str[128];
    int		index = 0;
    int		index_vert = 0;
    int		oldnbmess = 0;

    if( !strcmp( config_file, "not-defined") )
        sprintf(config_file, "%s/.wmpop3rc", getenv("HOME"));

    if( readConfigFile(config_file) == -1){
        exit(0);
    }
    pc =pop3Create();     /* initialize Pop3 ADT */

    /* Set up timer for checking mail */
    startTime     = time(0);  
    nextCheckTime = 0; /* Make 0, so it checks for mail on start */
    
    
    createXBMfromXPM(wminet_mask_bits, wmpop3_xpm
		     , wminet_mask_width, wminet_mask_height);
    
    openXwindow(argc, argv, wmpop3_xpm, wminet_mask_bits
		, wminet_mask_width, wminet_mask_height);
    
    AddMouseRegion(0, 19, 49, 30, 58); /* check button */
    AddMouseRegion(1, 46, 49, 50, 58 ); /* autocheck button   */
    AddMouseRegion(2, 53, 49, 57, 58 ); /* switch display button  */
    AddMouseRegion(3, 5, 49, 16, 58 ); /* delete button */ 
    AddMouseRegion(4, 33, 49, 44, 52); /* top arrow */
    AddMouseRegion(5, 33, 54, 44, 58); /* botton arrow */
    /* add the mouse regions for each mail */
    for (i = 0; i < NB_LINE; i++) {
      AddMouseRegion(6 + i, 9, (i*6) + TOP_MARGIN, 
		     58, (i*6) + TOP_MARGIN + CHAR_HEIGHT);
    }
    
    
    /* Check if Autochecking is on or off */
    if(autoChecking == NO ){
      copyXPMArea(142, 38, 7, 11, 45, 48);
    }
    else {
      copyXPMArea(142, 49, 7, 11, 45, 48);
    }
    
    RedrawWindow();

    
    while (1) {
      if( (((time(0) > nextCheckTime) || (nextCheckTime == 0))
	   && ( autoChecking == YES)) 
	  || (forcedCheck == YES)){ 
	/* clear the display */
	copyXPMArea(72, 4, 3, 42, LEFT_MARGIN, 4);
	copyXPMArea(72, 4, 50, 42, 9, 4 );
	BlitString("connect", 10, TOP_MARGIN);
	RedrawWindow();
	if(pop3MakeConnection(pc,popserver,serverport) == -1){
	  mailWaiting = -1;
	}
	if (-1 != mailWaiting) {
	  BlitString("login", 10, (6*2) + TOP_MARGIN);
	  RedrawWindow();
	  if (pop3Login(pc, username,password) == -1 ) 
	    mailWaiting = -1;
	}
	if (-1 != mailWaiting) {
	  BlitString("get mail", 10, (6*4) + TOP_MARGIN);
	  RedrawWindow();
	  if (pop3CheckMail(pc) == -1)
	    mailWaiting = -1;
	}
	pop3Quit(pc);
	if (-1 != mailWaiting) {
	  mailWaiting = pop3GetNumberOfUnreadMessages(pc);
	  if (oldnbmess != (totalMessages = pop3GetTotalNumberOfMessages(pc))){
	    oldnbmess = totalMessages;
	    if (totalMessages < MAX_NB_MSG)
	      index_vert = totalMessages - NB_LINE;
	    else 
	      index_vert = MAX_NB_MSG - NB_LINE;
	    if (index_vert < 0)
	      index_vert = 0;
	  }
	}
	nextCheckTime = time(0) + (mailCheckDelay * SEC_IN_MIN);
	if( forcedCheck == YES )
	  forcedCheck = NO;
	index = 0;
      }
      
      waitpid(0, NULL, WNOHANG);
      
      memset(str, 0, 128);
      if( mailWaiting == -1 ){
	/* we don't clear the display so that we know what was the error */
	/* Error connecting to pop server */
	BlitString(":error:", 10, (6*6) + TOP_MARGIN);
      }
      else {
	/* clear the display */
	copyXPMArea(72, 4, 3, 42, LEFT_MARGIN, 4);
	copyXPMArea(72, 4, 50, 42, 9, 4 );
	if (newMessagesOnly == YES ){
	  /* Show messages waiting */
	  sprintf(str, "mesg: %2d", totalMessages);
	  BlitString(str, 10, (int) (6*2) + TOP_MARGIN);
	  for (i =0, j = 0; i < totalMessages && i < MAX_NB_MSG; i++)
	    j += pc->mails[i].todelete;
	  sprintf(str, "sel.: %2d", j);
	  BlitString(str, 10, (int) (6*4) + TOP_MARGIN);	  
	}
	else {
	  if (0 == totalMessages) {
	    BlitString("No  Mesg", 10, (int)(6*2) + TOP_MARGIN);
	  }
	  else {
	    for (i = index_vert; i < (NB_LINE + index_vert) && 
		   i < MAX_NB_MSG && i < totalMessages; i++) {
	      build_line(str, i, index, pc);
	      BlitString(str, 10, ((i - index_vert) * 6) + TOP_MARGIN);
	      /* display the index */
	    }
	    display_index(index_vert, totalMessages, pc);
	  }
	  index++;
	}
      }
      
      RedrawWindow();
      
      /* X Events */
      while (XPending(display)){
	XNextEvent(display, &Event);
	switch (Event.type)
	  {
	  case Expose:
	    RedrawWindow();
	    break;
	  case DestroyNotify:
	    XCloseDisplay(display);
	    exit(0);
	    break;
	  case ButtonPress:
	    i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
	    buttonStatus = i;
	    if (buttonStatus == i && buttonStatus >= 0){
	      switch (buttonStatus){
	      case 0 : /* check button pressed */
		copyXPMArea(128,38 ,14 ,11 ,18 ,48 );
		break;
	      case 1 : /* autocheck button pressed */
		break;
	      case 2: /* switch display button pressed */
		break;
	      case 3: /* delete button pressed */
		copyXPMArea(127, 15, 14, 11, 4, 48);
		break;
	      case 4: /* top arrow button pressed */
		break;
	      case 5: /* bottom arrow button pressed */
		break;
	      default:
		break;
	      }
	      RedrawWindow();
	    }
	    break;
	  case ButtonRelease:
	    i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
	    
	    if (buttonStatus == i && buttonStatus >= 0){
	      switch (buttonStatus){
	      case 0 :   /* check button */
		copyXPMArea(128,27 ,14 ,11 ,18 ,48 );	
		forcedCheck = YES;
		break;
	      case 1 :   /* autocheck Button */
		if (autoChecking == YES){
		  autoChecking = NO;
		  copyXPMArea(142, 38, 7, 11, 45, 48);		  
		}
		else {
		  autoChecking = YES;
		  copyXPMArea(142, 49, 7, 11, 45, 48);
		}
		break;
	      case 2:  /* switch display Button */
		index = 0;
		/* change view on # of messages */
		if( newMessagesOnly == YES ) {
		  newMessagesOnly = NO;
		  copyXPMArea(149,38 , 7 , 11, 52, 48);
		} 
		else {
		  newMessagesOnly = YES;
		  copyXPMArea(149,49 , 7 , 11, 52, 48);
		}
		break;
	      case 3: /* delete button */
		copyXPMArea(143, 15, 14, 11, 4, 48);
		for (i =0, j = 0; i < totalMessages && i < MAX_NB_MSG; i++)
		  j += pc->mails[i].todelete;
		if (j) {
		  /* clear the display */
		  copyXPMArea(72, 4, 3, 42, LEFT_MARGIN, 4);
		  copyXPMArea(72, 4, 50, 42, 9, 4 );
		  mailWaiting = DeleteMail(pc);
		  if (mailWaiting < 0) {
		    BlitString(":error:", 10, 6*3 + TOP_MARGIN);
		    RedrawWindow();
		  }
		  else if (mailWaiting > 0) {
		    sprintf(str, "fail: %2d", mailWaiting);
		    BlitString(str, 10, 6*6 + TOP_MARGIN);
		    RedrawWindow();
		    sleep (1);
		  }
		  forcedCheck = YES;
		}
		break;
	      case 4: /* top arrow button pressed */
		index_vert--;
		if (0 > index_vert)
		  index_vert = 0;
		break;
	      case 5: /* bottom arrow button pressed */
		if (totalMessages > NB_LINE) {
		  index_vert++;
		  if (index_vert + NB_LINE > MAX_NB_MSG) 
		    index_vert = MAX_NB_MSG - NB_LINE;
		  if (index_vert + NB_LINE > totalMessages) 
		    index_vert = totalMessages - NB_LINE;
		}
		break;
	      default:
		/* don't do anything if we are not in the correct view mode */
		if (newMessagesOnly == NO) {
		  if ((5 < buttonStatus) && (buttonStatus <= 5 + NB_LINE)) {
		    if ((buttonStatus - 6 + index_vert) < totalMessages)
		      pc->mails[buttonStatus - 6 + index_vert].todelete =
			1 - pc->mails[buttonStatus - 6 + index_vert].todelete;
		  }
		}
		break;
	      }
	    }
	    RedrawWindow();
	    buttonStatus = -1;
	    break;
	  }
      }
	usleep(100000L);
    }
}


/*
 * usage : Prints proper command parameters of wmpop3.
 */
void usage(void)
{
  fprintf(stderr, "\nWMPop3 - Scott Holden <scotth@thezone.net>\n\n");
  fprintf(stderr, "usage:\n");
  fprintf(stderr, "    -display <display name>\n");
  fprintf(stderr, "    -geometry +XPOS+YPOS      initial window position\n");
  fprintf(stderr, "    -c <filename>             use specified config file\n");
  fprintf(stderr, "    -h                        this help screen\n");
  fprintf(stderr, "    -v                        print the version number\n");
  fprintf(stderr, "\n");
}

/*
 * printversion : This function is used to print the version info
 *                to standard output.
 */
void printversion(void)
{
  fprintf(stderr, "wmpop3 v%s\n", WMPOP3_VERSION);
}

// Blits a string at given co-ordinates
void BlitString(char *name, int x, int y)
{
  int		i;
  int		c;
  int		k;
  int		row;
  int		len;
  
  len = strlen(name);
  /* delete what was there before */
  copyXPMArea(15 * CHAR_WIDTH, NUMBERS, CHAR_WIDTH * len, CHAR_HEIGHT, x, y);
  for (i = 0, k = x; name[i]; i++, k += CHAR_WIDTH + 1) {
    c = toupper(name[i]); 
    if (c >= 'A' && c <= 'Z') {   /* its a letter */
      c -= 'A';
      row = LETTERS;
    }
    else if (c >= '0' && c <= '9') {   /* its a number */
      c -= '0';
      row = NUMBERS;
    }
    else switch(c) {
    case '-':
      c = 26;
      row = LETTERS;
      break;
    case '*':
      c = 27;
      row = LETTERS;
      break;
    case ':': 
      c = 10;
      row = NUMBERS;
      break;
    case '/':
      c = 11;
      row = NUMBERS;
      break;
    case '@':
      c = 12;
      row = NUMBERS;
      break;
    case ' ':
      c = 13;
      row = NUMBERS;
      break;
    case '.':
      c = 28;
      row = LETTERS;
      break;
    default:
      c = 14;
       row = NUMBERS;
       break;
    }
    copyXPMArea(c * CHAR_WIDTH, row, CHAR_WIDTH, CHAR_HEIGHT, k, y);
  }
}


/* Blits number to give coordinates...*/

void BlitNum(int num, int x, int y, int todelete)
{
  if (MAX_NB_MSG < num)
    return ;
  if (todelete)
    num += 19;
  copyXPMArea(((num - 1) * (SN_CHAR_W + 1)) + 1, SMALL_NUM, 
	      SN_CHAR_W, CHAR_HEIGHT + 1, x,y);
}
    

int  readConfigFile( char *filename ){
    char buf[256];
    char temp[32];
    char *ptr = 0;
    FILE *fp;

    if( (fp = fopen( filename, "r")) == 0 ){
        sprintf(config_file, "%s/.wmpop3rc", getenv("HOME"));
        printf("-Config file does not exit : %s\n",config_file);
        printf("+Trying to create new config file.\n");
        if((fp = fopen(config_file,"w")) == 0){
            printf("-Error creating new config file\n");
            return -1;
        }
        fprintf(fp,"# Replace all < > with appropriate data\n#\n");
        fprintf(fp,"popserver          < pop3 server name >\n");
        fprintf(fp,"port               110    # default port\n");
        fprintf(fp,"username           < pop3 login name  >\n");
        fprintf(fp,"password           < pop3 password   >\n");
        fprintf(fp,"autochecking       0      # 1 enables, 0 disables\n");
        fprintf(fp,"mailcheckdelay     10     # default mail check time in minutes\n");
        fprintf(fp,"viewallmessages    0      # 0 Shows both read and unread messages\n");
        fprintf(fp,"#                           and 1 shows only unread messages\n");
        fprintf(fp,"mailclient         pine  # default mail client\n");
        printf("+New config file created : ~/.wmpop3rc\n\n");
        printf("+ ~/.wmpop3rc must be configured before running wmpop3.\n");
        fclose(fp);
        return -1;
    }

    while( fgets(buf, 256, fp) != 0){

        ptr = strtok( buf," \n" );

        if( ( ptr != 0) && (ptr[0] != '#') ){
            if( !strcmp(ptr, "username") ){
                ptr = strtok( 0, " \n");
                if( ptr == 0){
                    printf("Invalid UserName.\n");
                    return -1;
                }
                strcpy(username, ptr); 
            }
            else if( !strcmp( ptr, "password") ){
                ptr = strtok( 0, " \n");
                if( ptr == 0){
                    printf("Invalid password.\n");
                    return -1;
                }
                strcpy(password, ptr); 
            }
            else if( !strcmp( ptr, "popserver") ){
                ptr = strtok( 0, " \n");
                if( ptr == 0){
                    printf("Invalid popserver address.\n");
                    return -1;
                }
                strcpy(popserver, ptr); 
            }
            else if( !strcmp( ptr, "mailclient") ){
                ptr = strtok( 0, " \n");
                if( ptr == 0){
                    printf("Invalid mailclient.\n");
                    return -1;
                }
                strcpy(mailclient, ptr); 
            }
            else if( !strcmp( ptr, "port") ){
                ptr = strtok( 0, " \n");
                if( ptr == 0){
                    printf("Invalid popserver port number.\n");
                    return -1;
                }
                if( sscanf(ptr,"%[0123456789]",temp) == 0)
                    serverport = 110;
                else
                    serverport = atoi(temp);
            }
            else if( !strcmp( ptr, "viewallmessages") ){
                ptr = strtok( 0, " \n");
                if( ptr == 0){
                    printf("Invalid number. ( viewallmessages )\n");
                    return -1;
                }
                if( sscanf(ptr,"%[0123456789]",temp) != 0)
                    newMessagesOnly = atoi(temp);
            }
            else if( !strcmp( ptr, "mailcheckdelay") ){
                ptr = strtok( 0, " \n");
                if( ptr == 0){
                    printf("Invalid delay time.\n");
                    return -1;
                }
                if( sscanf(ptr,"%[0123456789]",temp) != 0)
                    mailCheckDelay = atoi(temp);
            }
            else if( !strcmp( ptr, "autochecking") ){
                ptr = strtok( 0, " \n");
                if( ptr == 0){
                    printf("Invalid value.\n");
                    return -1;
                }
                if( sscanf(ptr,"%[0123456789]",temp) != 0)
                    autoChecking = atoi(temp);
            }
            else
                printf("Unknown indenifier : %s\n",ptr);
        }
    }
    fclose(fp);
    return 0;
}

